#!/usr/bin/python

__doc__ = """The module contains the father classes of all bot's with the standard way to make a move. 

Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE."""

import gamefield
import playerstat
import results
import time
import os

class botf():
    """The father of all bots.

The class gives a framework which loads all necessary data for the child classes. 
It will load the gamefield, player and opponent data. Further it will write the move
that a childclass has calclulated. 
botf will also measure the time a childclass needs to calculate its move.

Attention: The childclass have to overwrite the function calcmove"""

    def __init__(self,path):
        """Initializer

    Creates the necessary objects for childclasses from the files laying under path. 
    """
        self.path = path
        self.field = gamefield.playground(os.path.join(self.path,"gamefield.dat"))
        self.pstats = playerstat.stat(os.path.join(self.path,"player.dat"))
        self.ostats = playerstat.stat(os.path.join(self.path,"opponent.dat"))
	self.move = (-1,-1,-1,-1)

    def giveextramove(self,result):
        """Specialfunction for lilac

    The function calculates from the playerstats and the given result values if the
    move will be generate an extra move.
    """
        if self.pstats.values['L'] + result.values['L'] >= 15:
	    return True
	else:
	    return False

    def isdeadlyimpact(self,result):
        """Specialfunction for player

    The function will check if the given result regarding to the opponent stats
    are good enough to kill him.
    """
        remaininglife = self.ostats.values['Shield'] + self.ostats.values['Life'] - result.calcdamage(self.pstats)
	if remaininglife <= 0:
	    return True
	else:
	    return False
	
    def isplayerkill(self,oresult,presult):
        """Specialfunction for player

    Equal to isdeadlyimpact but will check if the presult and the playerstat 
    will kill the player.
    """
        remaininglife = self.pstats.values['Shield'] + self.pstats.values['Life'] + presult.values['B'] - oresult.calcdamage(self.ostats)
	if remaininglife <= 0:
	    return True
	else:
	    return False

    def makemove(self):
        """Frameworfunction

    Function call the calcmove method of the actual object. Further it measures 
    the time used to run this function and saves the result.
    """
        timestart = time.time()
	self.calcmove()
	timeend = time.time()
	self.duration = timeend - timestart
    
    def writemove(self):
        """Frameworkfunction

    Will take the given self.move and will form it to the strange index format of the gameengine.
    """
        movestring = "%d %d %d %d" % (self.move[1],-self.move[0] + 9,self.move[3], - self.move[2] + 9)
        open(os.path.join(self.path,"result.dat"),mode = 'w').write(movestring)

    def __repr__(self):
       """Frameworkfunction

    Repression of the object. In this case only the actual move duration is printed.
    """
       string = "%s move duration: %f" % (self.__class__.__name__, self.duration)
       return string

class botfparam(botf):
    """Class with multiplier paramters

The class enables the bots to use a set of 'optimal' multipliers to
calculate the value of a move.
"""
    def __init__(self,path):
        """Initializer

    The init sets the momentan opitmal values for the multipliers
    """
        botf.__init__(self,path)
	self.param = {'fpshield' : 0.3,
	              'fplife'   : 0.6,
		      'foshield' : 0.3,
		      'folife'   : 0.3,
		      'fblue'    : 0.9,
                      'flilac'   : 0.9
		     }

    def calcfblue(self):
        """Unused
    """
        return 0.0

    def calcflilac(self):
        """Unused
    """
        return 0.0

class botftrainer(botfparam):
    """Class to test the optimal multiplier paramters

This class can be controled from outside which paramters are used as multipliers.
"""
    def __init__(self,path):
        """Initializer

    The function loads the multplier values from a file and set them in the dictionary.
    """
        botfparam.__init__(self,path)
        trainparam = open(os.path.join(self.path,"train.dat")).readlines()

	for param in trainparam:
	    param = param.split('=')
	    self.param[param[0]] = float(param[1])
	    print self.param



if __name__ == "__main__":
    test = botf(os.getcwd())
    test.makemove()
    test.writemove()

    if verbose: print test
